//alignAndDistribute_=0mm.jsx　2022.09.06（z-）ver_e

//id_distribute_=_0mm.jsx の強化版。整列・規定間隔値で分布を一括処理。
//整列・分布タイプ、間隔値をスクリプトのファイル名で指定。
//ファイル名は "〜_[整列タイプ][基準位置](間隔値)(_リピート).jsx" のように
//半角アンダースコアで区切って必要なだけ入れて構成してください。

//ファイル名ルール（以下アルファベットは小文字可）
//整列 は "整" または "A" (Align) のいずれか
//分布 は "分" または "D" (Distribute)
//上 は "上" または "天" またはTopの "T "
//下 は "下" または "地" またはBottomの "B"
//左 は "左" またはLeftの "L"
//右 は "右" またはRightの "R"
//水平方向の中央 は "水" またはHorizontalの "H"
//垂直方向の中央は "垂" またはVerticalの"V"
///////
//等間隔に分布のみ、水平分布は "!!"、垂直分布は "=" と表記する
//ファイル名先頭にはユーザー任意の文字列を入れてOK 入れなくてもOK

//////　例　///////
// "hoge_整上_整左.jsx"　（左上揃え）
// "fuga_AR_=2mm.jsx"　（右揃え＆水平方向に2mm間隔で分布）
// "piyo_分T40mm_!!8H.jsx"　（左基準40mm分布＆垂直方向に8歯間隔で分布）
//※左から順に処理します

//2017.06.27　ver_b　Windows環境ではファイル名に"||"を使用できないとの事で"!!"に変更
//2017.09.22　ver_c　キーオブジェクト未指定の際の挙動を改善
//2022.08.22〜 ver_d　整列、分布を複合指定可能に
//2022.09.06〜 ver_e　キーブジェクトなしの等間隔で動かなかったのを改修

//ドキュメントが開かれていて選択オブジェクトが2つ以上あるか見ている
try{
    app.selection[1].name;
    }
catch(e){
    exit();
    }

var doc = app.activeDocument;
var sel = app.selection;

//ファイル名の調査

//アンダースコアと拡張子で区切る
//var fileName = decodeURI($.fileName).match(/_(=||\!\!)_\d*\.?\d*?[a-z]*(?=\.jsx$)/);
//var fileName = decodeURI(app.activeScript.name);
var fileName = decodeURI($.fileName).replace(/^.+[\/\\](?!=[\/\\]+$)/, "");
var reg = fileName.match(/[^_]+(?=(_|\.jsx))/gi);
if(reg == null){
    alert("ファイル名のエラーです\n\n命名ルールはスクリプト冒頭のコメント部分または配布サイトを参照してください");
    exit();
    }
//各パートの調査
var ary = [], wrongAry = [];
for(var i = 0; i < reg.length; i++){
    if(reg[i].match(/^[整A][上天T下地B左L右R水H垂V]$/i)){
        ary.push(reg[i]);
        continue;
        }
    if(reg[i].match(/^[分D][上天T下地B左L右R水H垂V]((\d*\.)?\d+[A-Z]*)?$/i)){
        ary.push(reg[i]);
        continue;
        }
    if(reg[i].match(/^(\=|\!\!)(\d*\.)?\d+[A-Z]*$/i)){
        ary.push(reg[i]);
        continue;
        }
    if(i != 0){
        wrongAry.push(reg[i]);
        }
    }

if(ary.length == 0){
    alert("命名ルールに則った文字列が見つかりませんでした\n\n命名ルールはスクリプト冒頭のコメント部分または配布サイトを参照してください");
    exit();
    }

if(ary.length && wrongAry.length){
    alert(wrongAry.join("\n") + "は命名ルール外の表記と見なされました");
    }

//alert(ary.join("\n")); exit();

//こまかい解析とパラメータのオブジェクト化_書き方ださい…
var prms = [];
for(i = 0; i < ary.length; i++){
    var prm = {};
    if(ary[i].match(/^[整A]/i)){
        prm.type = "align";
        if(ary[i].match(/[上天T]/)){
            prm.opt = AlignOptions.TOP_EDGES;
            }
        if(ary[i].match(/[下地B]/)){
            prm.opt = AlignOptions.BOTTOM_EDGES;
            }
        if(ary[i].match(/[左L]/)){
            prm.opt = AlignOptions.LEFT_EDGES;
            }
        if(ary[i].match(/[右R]/)){
            prm.opt = AlignOptions.RIGHT_EDGES;
            }
        if(ary[i].match(/[水H]/)){
            prm.opt = AlignOptions.HORIZONTAL_CENTERS;
            }
        if(ary[i].match(/[垂V]/)){
            prm.opt = AlignOptions.VERTICAL_CENTERS;
            }
        } else {
        prm.type = "distribute";
        if(ary[i].match(/[上天T]/)){
            prm.opt = DistributeOptions.TOP_EDGES;
            }
        if(ary[i].match(/[下地B]/)){
            prm.opt = DistributeOptions.BOTTOM_EDGES;
            }
        if(ary[i].match(/[左L]/)){
            prm.opt = DistributeOptions.LEFT_EDGES;
            }
        if(ary[i].match(/[右R]/)){
            prm.opt = DistributeOptions.RIGHT_EDGES;
            }
        if(ary[i].match(/[水H]/)){
            prm.opt = DistributeOptions.HORIZONTAL_CENTERS;
            }
        if(ary[i].match(/[垂V]/)){
            prm.opt = DistributeOptions.VERTICAL_CENTERS;
            }
        if(ary[i].match(/^\=/)){
            prm.opt = DistributeOptions.VERTICAL_SPACE;
            }
        if(ary[i].match(/^\!\!/)){
            prm.opt = DistributeOptions.HORIZONTAL_SPACE;
            }
        }
    var val = ary[i].match(/(\d*\.)?\d+[A-Z]*$/i);
    if(val){
        prm.val = val[0];
        }
    prms.push(prm);
    }

app.doScript(pepsi, ScriptLanguage.JAVASCRIPT, [], UndoModes.FAST_ENTIRE_SCRIPT); 

function pepsi (){
    for(var i = 0; i < prms.length; i++){
        var prm = prms[i];
        try{
            if(prm.type == "align"){
                doc.align(
                sel, 
                prm.opt, 
                doc.selectionKeyObject ? AlignDistributeBounds.KEY_OBJECT : AlignDistributeBounds.ITEM_BOUNDS, 
                doc.selectionKeyObject ? app.selectionKeyObject : undefined
                );
            } else{
                doc.distribute(
                sel, 
                prm.opt, 
                doc.selectionKeyObject ? AlignDistributeBounds.KEY_OBJECT : undefined,  
                true, 
                prm.val ? prm.val : 0, 
                doc.selectionKeyObject ? app.selectionKeyObject : undefined
                );
                }
        } catch(e){
            //alert("エラー：\nたぶん単位の表記が無効です");
            alert(e);
            }
        }
    }